// Ventilateur : application Librithèque I3S2

#define ENABLE 2          // Commande autorisation
#define MOTEUR 3          // Vitesse du moteur (analogique)
#define TRIGGER 4         // broche pin trigger
#define ECHO 5            // broche pin echo
#define BP 6              // broche bouton poussoir
#define DISTANCE_VMAX 100 // distance à laquelle on laisse la vitesse maximum

int distance;         // entier pour la distance en cm
int etatBP;           // entier pour état du bouton
int vitesse;          // entier pour la distance (0 arrêté, 255 vitesse max)
int distanceMin;      // entier pour la distance de sécurité 

void setup() {
  pinMode(TRIGGER,OUTPUT);      // pin trigger définie en sortie
  pinMode(ECHO, INPUT);         // pin écho définie en entrée
  pinMode(ENABLE, OUTPUT);      
  pinMode(MOTEUR, OUTPUT);
  pinMode(BP,INPUT_PULLUP);     // pin BP (6) en mode entrée avec pullup
  Serial.begin(9600);           // initialisation liaison série pour affichage message
  vitesse = 0;                  // au départ le moteur est arrêté
  etatBP = 1;                   // au départ le bouton n'est pas appuyé
  digitalWrite(ENABLE, HIGH);   // Autorisation rotation
  analogWrite(MOTEUR, vitesse); // Vitesse nulle
  digitalWrite(TRIGGER, LOW);   // RAZ pin trigger 

  // Etalonnage distance de sécurité
  //--------------------------------

  // attente appui sur le BP (BP à 0 quand appuyé)
  while (etatBP == 1) {
    etatBP = digitalRead(BP);
    Serial.print("Attente BP ");
    Serial.println(etatBP);
    delay (200);
  }// while etatBP

  // bouton appuyé, lecture de la distance 
  do{
    etatBP = digitalRead(BP);
    distanceMin = lireDistance();
    Serial.print("Etalonnage : distance lue ");
    Serial.println(distanceMin);     
    delay(200);
  }
  while (etatBP == 0);
  // A la sortie, distanceMin contient la dernière valeur lue avant le relachement du poussoir
}// setup

void loop() {
  distance = lireDistance();    // mesure de la distance
  // calcul de la vitesse
  if (distance > distanceMin)
  {
    vitesse = (255/(DISTANCE_VMAX - distanceMin)) * (distance - distanceMin);
    if (vitesse > 255) vitesse = 255;
  }
  else vitesse = 0;             // si trop près, arrêt

  analogWrite(MOTEUR, vitesse); 

  Serial.print("Distance = ");
  Serial.print(distance);
  Serial.print(" -> vitesse = ");
  Serial.println(vitesse);
  delay(500);
}

int lireDistance()
{
  long duree;                   // entier long pour la durée en microsecondes

  // sortie trigger: impulsion de 10µs 
  digitalWrite(TRIGGER, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGGER, LOW);

  // acquisition de la durée de l'impulsion 
  duree = pulseIn(ECHO, HIGH);
  // calcul de la distance : 343 m/s  343/1000000 m/µs = (343 * 100)/1000000 cm/µs 
  return((duree * 0.0343) / 2);  //divisée par 2 pour Aller/Retour
}
